@@ -59,15 +59,18 @@
 #
 # --recipe name
 #   specify which build recipe to use.  Not all recipes are supported on all targets.
-#       -gcc-5.2.0          (mingw-w64 project)
+#     * -gcc-5.2.0          (mingw-w64 project)
 #       -gcc-7.1.0          (mingw-w64 project)
 #       -gcc-7.1.0r0        (mingw-w64 project)
 #       -gcc-7.1.0r2        (mingw-w64 project)
 #       -gcc-7.3.0          (mingw-w64 project)
-#       -gcc-8.1.0          (mingw-w64 project)
-#       -winlibs-gcc-8.4.0  (winlibs mingwrt 7.0.0r1)
-#       -winlibs-gcc-9.3.0  (winlibs mingwrt 7.0.0r3 - sjlj)
-#       -winlibs-gcc-9.3.0  (winlibs mingwrt 7.0.0r4)
+#     * -gcc-8.1.0          (mingw-w64 project)
+#     * -gcc-11.2.0         (mingw-w64 project)
+#       -gcc-12.2.0         (mingw-w64 project)
+#     * -winlibs-gcc-9.3.0  (winlibs mingwrt 7.0.0r3 - sjlj)
+#       -winlibs-gcc-11.2.0 (winlibs mingwrt 10.0.0r1 - msvcrt)
+#       -winlibs-gcc-11.3.0 (winlibs mingwrt 10.0.0r3 - msvcrt)
+#       -winlibs-gcc-11.4.0 (winlibs mingwrt 11.0.0r1 - msvcrt)
 #       -winlibs-gcc-10.2.0 (winlibs mingwrt 8.0.0r8)
 #       -winlibs-gcc-10.3.0 (winlibs mingwrt 8.0.0r1)
 #       -equation-gcc-8.3.0 (Equation - experimental)
@@ -375,6 +378,30 @@ get_winlibs_toolchain() {
 	fi
 
 	case "$named_recipe" in
+	-winlibs-gcc-11.4.0)
+		gccversion=11.4.0
+		llvmversion=
+		mingwruntime=11.0.0
+		mingwbuildsrev=r1
+		winlibsdir=$gccversion-$mingwruntime-msvcrt-$mingwbuildsrev
+		file=winlibs-$arch-posix-$default_eh-gcc-$gccversion-mingw-w64msvcrt-$mingwruntime-$mingwbuildsrev.7z
+		;;
+	-winlibs-gcc-11.3.0)
+		gccversion=11.3.0
+		llvmversion=14.0.3
+		mingwruntime=10.0.0
+		mingwbuildsrev=r3
+		winlibsdir=$gccversion-$llvmversion-$mingwruntime-msvcrt-$mingwbuildsrev
+		file=winlibs-$arch-posix-$default_eh-gcc-$gccversion-mingw-w64msvcrt-$mingwruntime-$mingwbuildsrev.7z
+		;;
+	-winlibs-gcc-11.2.0)
+		gccversion=11.2.0
+		llvmversion=
+		mingwruntime=10.0.0
+		mingwbuildsrev=r1
+		winlibsdir=$gccversion-$mingwruntime-msvcrt-$mingwbuildsrev
+		file=winlibs-$arch-posix-$default_eh-gcc-$gccversion-mingw-w64msvcrt-$mingwruntime-$mingwbuildsrev.7z
+		;;
 	-winlibs-gcc-10.3.0)
 		gccversion=10.3.0
 		llvmversion=11.1.0
@@ -391,15 +418,6 @@ get_winlibs_toolchain() {
 		winlibsdir=$gccversion-$mingwruntime-$mingwbuildsrev
 		file=winlibs-$arch-posix-$default_eh-gcc-$gccversion-mingw-w64-$mingwruntime-$mingwbuildsrev.7z
 		;;
-#	-winlibs-gcc-9.3.0)
-#		gccversion=9.3.0
-#		llvmversion=10.0.0
-#		mingwruntime=7.0.0
-#		mingwbuildsrev=r3
-#		toolchain=winlibs
-#		winlibsdir=$gccversion-$llvmversion-$mingwruntime-$mingwbuildsrev
-#		file=winlibs-$arch-posix-$default_eh-gcc-$gccversion-llvm-$llvmversion-mingw-w64-$mingwruntime-$mingwbuildsrev.7z
-#		;;
 	-winlibs-gcc-9.3.0)
 		gccversion=9.3.0
 		llvmversion=
@@ -408,14 +426,6 @@ get_winlibs_toolchain() {
 		winlibsdir=$gccversion-$mingwruntime-sjlj-$mingwbuildsrev
 		file=winlibs-mingw-w64-$arch-$gccversion-$mingwruntime-$mingwbuildsrev-sjlj.7z
 		;;
-	-winlibs-gcc-8.4.0)
-		gccversion=8.4.0
-		llvmversion=
-		mingwbuildsrev=r1
-		mingwruntime=7.0.0
-		winlibsdir=$gccversion-$mingwruntime-$mingwbuildsrev
-		file=mingw-w64-$arch-$gccversion-$mingwruntime.7z
-		;;
 	*)
 		echo "get_winlibs_toolchain(): invalid recipe $named_recipe"
 		exit 1
@@ -439,27 +449,27 @@ get_mingww64_toolchain() {
 	toolchain=mingw-w64
 	
 	case "$named_recipe" in
-	-gcc-8.1.0)
+	-mingw-w64-gcc-8.1.0|-gcc-8.1.0)
 		gccversion=8.1.0
 		mingwbuildsrev=rev0
 		mingwruntime=v6
 		;;
-	-gcc-7.3.0)
+	-mingw-w64-gcc-7.3.0|-gcc-7.3.0)
 		gccversion=7.3.0
 		mingwbuildsrev=rev0
 		mingwruntime=v5
 		;;
-	-gcc-7.1.0r2)
+	-mingw-w64-gcc-7.1.0r2|-gcc-7.1.0r2)
 		gccversion=7.1.0
 		mingwbuildsrev=rev2
 		mingwruntime=v5
 		;;
-	-gcc-7.1.0|-gcc-7.1.0r0)
+	-mingw-w64-gcc-7.1.0|-mingw-w64-gcc-7.1.0r0|-gcc-7.1.0)
 		gccversion=7.1.0
 		mingwbuildsrev=rev0
 		mingwruntime=v5
 		;;
-	-gcc-5.2.0)
+	-mingw-w64-gcc-5.2.0|-gcc-5.2.0)
 		gccversion=5.2.0
 		mingwbuildsrev=rev0
 		mingwruntime=v4
@@ -484,6 +494,49 @@ get_mingww64_toolchain() {
 	7z x "../input/MinGW-w64/$file" > /dev/null
 }
 
+get_nixman_toolchain() {
+	bits="$1"
+	arch="$2"
+	toolchain=mingw-w64
+	default_eh=sjlj
+	default_rt=rt
+
+	case "$named_recipe" in
+	-mingw-w64-gcc-12.2.0|-gcc-12.2.0)
+		gccversion=12.2.0
+		mingwbuildsrev=rev2
+		mingwruntime=v10
+		if [ "$bits" = "32" ]; then
+			default_eh=dwarf
+		else
+			default_eh=seh
+		fi
+		default_rt=msvcrt-rt
+		;;
+	-mingw-w64-gcc-11.2.0|-gcc-11.2.0)
+		gccversion=11.2.0
+		mingwbuildsrev=rev1
+		mingwruntime=v9
+		;;
+	*)
+		echo "get_nixman_toolchain(): invalid toolchain version $named_recipe"
+		exit 1
+	esac
+
+	# nixman
+	dir=$gccversion-rt_$mingwruntime-$mingwbuildsrev/
+	file=$arch-$gccversion-release-win32-$default_eh-${default_rt}_$mingwruntime-$mingwbuildsrev.7z
+	binUrl=https://github.com/niXman/mingw-builds-binaries/releases/download/$dir$file
+
+	srcfile=$gccversion-rt_$mingwruntime-$mingwbuildsrev.tar.gz
+	srcUrl=https://github.com/niXman/mingw-builds-binaries/archive/refs/tags/$srcfile
+
+	mkdir -p ./packages/mingw-w64
+	download "mingw-w64/$file"    $binUrl
+	download "mingw-w64/$srcfile" $srcUrl
+	7z x "../input/MinGW-w64/$file" > /dev/null
+}
+
 get_windows_toolchain() {
 	bits="$1"
 	arch="$2"
@@ -495,7 +548,10 @@ get_windows_toolchain() {
 	-equation-*)
 		get_equation_toolchain $bits $arch
 		;;
-	-gcc-*)
+	-mingw-w64-gcc-11.2.0|-gcc-11.2.0|-mingw-w64-gcc-12.2.0|-gcc-12.2.0)
+		get_nixman_toolchain $bits $arch
+		;;
+	-mingw-w64-gcc-*|-gcc-*)
 		get_mingww64_toolchain $bits $arch
 		;;
 	*)
@@ -517,9 +573,9 @@ dos)
 	}
 
 	djver=205
-	gccver=710
-	djgppgccversiondir=7
-	bnuver=229
+	gccver=930
+	djgppgccversiondir=9
+	bnuver=2351
 	gdbver=771
 	djpkg=current
 
@@ -532,9 +588,9 @@ dos)
 	# rest to complete the DJGPP install (usually no changes needed)
 	download_djgpp ${djpkg}/v2/ djdev${djver}
 
-	download_djgpp ${djpkg}/v2gnu/ fil41br2
-	download_djgpp ${djpkg}/v2gnu/ mak421b
-	download_djgpp ${djpkg}/v2gnu/ shl2011br2
+	download_djgpp ${djpkg}/v2gnu/ fil41br3
+	download_djgpp ${djpkg}/v2gnu/ mak43br2
+	download_djgpp ${djpkg}/v2gnu/ shl2011br3
 	download_djgpp ${djpkg}/v2gnu/ pth207b 
 
 	download_djgpp ${djpkg}/v2tk/ ls080b 
@@ -547,9 +603,9 @@ dos)
 
 	unzip -qo ../input/DJGPP/djdev${djver}.zip
 	
-	unzip -qo ../input/DJGPP/shl2011br2.zip
-	unzip -qo ../input/DJGPP/fil41br2.zip
-	unzip -qo ../input/DJGPP/mak421b.zip
+	unzip -qo ../input/DJGPP/shl2011br3.zip
+	unzip -qo ../input/DJGPP/fil41br3.zip
+	unzip -qo ../input/DJGPP/mak43br2.zip
 	unzip -qo ../input/DJGPP/pth207b.zip
 
 	unzip -qo ../input/DJGPP/ls080b.zip
@@ -559,7 +615,7 @@ dos)
 	unzip -qo ../input/DJGPP/gcc${gccver}b.zip
 	unzip -qo ../input/DJGPP/gpp${gccver}b.zip
 	
-	patch -p0 < ../djgpp-fix-pthread.patch
+	patch -p0 < ../djgpp-pthread-types.patch
 	;;
 win32)
 	get_windows_toolchain 32 i686
@@ -595,6 +651,7 @@ win32-mingworg)
 	download_extract_mingw gmp-5.1.2-1-mingw32-dll.tar.lzma
 	download_extract_mingw mpc-1.0.1-2-mingw32-dll.tar.lzma
 	download_extract_mingw mpfr-3.1.2-2-mingw32-dll.tar.lzma
+	download_extract_mingw libiconv-1.14-3-mingw32-dll.tar.lzma
 
 	# Add ddraw.h and dinput.h for FB's gfxlib2
 
@@ -616,8 +673,17 @@ win64)
 esac
 
 # choose a bootstrap source for the target
+# - the minimum needed to build the current release
 case $fbtarget in
-linux-arm|linux-aarch64)
+linux-arm)
+	# - 1.10.2 for arm is a little weird because it depended on changes 
+	#   in fbc to make the 1.10.2 release work automatically and bootstrap
+	#   on debian 12 with gcc 12.  Could bootstrap from older versions
+	#   but it requires some manual intervention to set compile options
+	BUILD_BOOTFBCFLAGS=DEFAULT_CPUTYPE_ARM=FB_CPUTYPE_ARMV7A_FP
+	bootfb_title=FreeBASIC-1.10.2-$fbtarget
+	;;
+linux-aarch64)
 	bootfb_title=FreeBASIC-1.07.2-$fbtarget
 	;;
 *)
@@ -660,7 +726,7 @@ linux*)
 esac
 
 case $fbtarget in
-win32|win64)
+win32)
 	# libffi sources https://github.com/libffi/libffi/releases/download/v3.3/libffi-3.3.tar.gz. 
 	libffi_title=libffi-3.3
 	libffi_package="${libffi_title}.tar.gz"
@@ -668,6 +734,25 @@ win32|win64)
 	echo "extracting $libffi_package"
 	tar xf "../input/$libffi_package"
 	;;
+
+win64)
+	# libffi sources: 
+	# - ftp://sourceware.org/pub/libffi/libffi-3.4.3.tar.gz 
+	# - https://github.com/libffi/libffi/releases/download/v3.4.4/libffi-3.4.4.tar.gz
+	libffi_version=3.4.4
+	libffi_title=libffi-${libffi_version}
+	libffi_package="${libffi_title}.tar.gz"
+
+	# sourware:
+	# libffi_dir="ftp://sourceware.org/pub/libffi/"
+
+	# github:
+	libffi_dir="https://github.com/libffi/libffi/releases/download/v${libffi_version}/"
+
+	download "$libffi_package" "${libffi_dir}${libffi_package}"
+	echo "extracting ${libffi_package}"
+	tar xf "../input/${libffi_package}"
+	;;
 esac
 
 dosbuild() {
@@ -717,6 +802,7 @@ EOF
 	mkdir -p fbc/bin/dos
 	cp bin/ar.exe bin/as.exe bin/gdb.exe bin/gprof.exe bin/ld.exe fbc/bin/dos/
 	cp bin/dxe3gen.exe fbc/bin/dos/
+	cp lib/dxe.ld fbc/lib/dos/dxe.ld 
 	cp lib/crt0.o lib/gcrt0.o lib/libdbg.a lib/libemu.a lib/libm.a fbc/lib/dos/
 	cp lib/libstdcxx.a fbc/lib/dos/libstdcx.a
 	cp lib/libsupcxx.a fbc/lib/dos/libsupcx.a
@@ -739,16 +825,16 @@ linuxbuild() {
 	echo
 	echo "bootstrapping normal fbc"
 	echo
-	make FBC=../$bootfb_title/bin/fbc FBSHA1=$FBSHA1
+	make FBC=../$bootfb_title/bin/fbc FBSHA1=$FBSHA1 $BUILD_BOOTFBCFLAGS
 	make install prefix=../tempinstall
 	echo
 	echo "rebuilding normal fbc"
 	echo
 	make clean-compiler
-	make FBC=../tempinstall/bin/fbc FBSHA1=$FBSHA1
+	make FBC=../tempinstall/bin/fbc FBSHA1=$FBSHA1 $BUILD_BOOTFBCFLAGS
 	cd ..
 
-	cd fbc && make bindist FBSHA1=$FBSHA1 && cd ..
+	cd fbc && make bindist FBSHA1=$FBSHA1 $BUILD_BOOTFBCFLAGS && cd ..
 	cp fbc/*.tar.* ../output
 	cp fbc/contrib/manifest/FreeBASIC-$fbtarget.lst ../output
 }
@@ -757,7 +843,7 @@ libffibuild() {
 
 	# on fbc 1.08.0 we changed the libffi version so
 	# don't use any cached files when building the packages
-	# commented out for future reference
+	# unless we explicitly ask to with --use-libffi-cache 
 	# do we already have the files we need?
 	if [ "$uselibfficache" = "Y" ]; then
 	if [ -f "../input/$libffi_title/$target$named_recipe/ffi.h" ]; then
@@ -907,7 +993,7 @@ windowsbuild() {
 			;;
 		esac
 		;;
-	-winlibs-gcc-8.4.0|-winlibs-gcc-9.3.0|-winlibs-gcc-10.2.0|-winlibs-gcc-10.3.0)
+	-winlibs-gcc-9.3.0|-winlibs-gcc-10.2.0|-winlibs-gcc-10.3.0|-winlibs-gcc-11.3.0)
 		# -winlibs-gcc-X.X is being built from winlibs and the binutils have a few dependencies
 		# copy these to the bin directory - they go with the executables and should
 		# not be used as general libraries
@@ -965,6 +1051,7 @@ windowsbuild() {
 		win32-mingworg)
 			cp bin/gdb.exe              fbc/bin/win32
 			cp bin/libgcc_s_dw2-1.dll   fbc/bin/win32
+			cp bin/libiconv-2.dll       fbc/bin/win32
 			cp bin/zlib1.dll            fbc/bin/win32
 			;;
 		win64)
